# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class ConfigurationQueriesTestResponse(Model):
    """ConfigurationQueriesTestResponse.

    :param target_condition_error: The errors from running the target
     condition query.
    :type target_condition_error: str
    :param custom_metric_query_errors: The errors from running the custom
     metric query.
    :type custom_metric_query_errors: dict[str, str]
    """

    _attribute_map = {
        "target_condition_error": {"key": "targetConditionError", "type": "str"},
        "custom_metric_query_errors": {"key": "customMetricQueryErrors", "type": "{str}"},
    }

    def __init__(
        self, *, target_condition_error: str = None, custom_metric_query_errors=None, **kwargs
    ) -> None:
        super(ConfigurationQueriesTestResponse, self).__init__(**kwargs)
        self.target_condition_error = target_condition_error
        self.custom_metric_query_errors = custom_metric_query_errors
