# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class ConfigurationContent(Model):
    """The configuration content for devices or modules on edge devices.

    :param device_content: The device configuration content.
    :type device_content: dict[str, object]
    :param modules_content: The modules configuration content.
    :type modules_content: dict[str, dict[str, object]]
    :param module_content: The module configuration content.
    :type module_content: dict[str, object]
    """

    _attribute_map = {
        "device_content": {"key": "deviceContent", "type": "{object}"},
        "modules_content": {"key": "modulesContent", "type": "{{object}}"},
        "module_content": {"key": "moduleContent", "type": "{object}"},
    }

    def __init__(
        self, *, device_content=None, modules_content=None, module_content=None, **kwargs
    ) -> None:
        super(ConfigurationContent, self).__init__(**kwargs)
        self.device_content = device_content
        self.modules_content = modules_content
        self.module_content = module_content
