# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class ManagedIdentity(Model):
    """ManagedIdentity.

    :param user_assigned_identity:
    :type user_assigned_identity: str
    """

    _attribute_map = {"user_assigned_identity": {"key": "userAssignedIdentity", "type": "str"}}

    def __init__(self, *, user_assigned_identity: str = None, **kwargs) -> None:
        super(ManagedIdentity, self).__init__(**kwargs)
        self.user_assigned_identity = user_assigned_identity
