# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class DeviceRegistryOperationWarning(Model):
    """The device registry operation warning details.

    :param device_id: The unique identifier of the device.
    :type device_id: str
    :param warning_code: The warning code. Possible values include:
     'DeviceRegisteredWithoutTwin'
    :type warning_code: str or ~protocol.models.enum
    :param warning_status: The details of the warning.
    :type warning_status: str
    """

    _attribute_map = {
        "device_id": {"key": "deviceId", "type": "str"},
        "warning_code": {"key": "warningCode", "type": "str"},
        "warning_status": {"key": "warningStatus", "type": "str"},
    }

    def __init__(
        self, *, device_id: str = None, warning_code=None, warning_status: str = None, **kwargs
    ) -> None:
        super(DeviceRegistryOperationWarning, self).__init__(**kwargs)
        self.device_id = device_id
        self.warning_code = warning_code
        self.warning_status = warning_status
