# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class ConfigurationQueriesTestInput(Model):
    """ConfigurationQueriesTestInput.

    :param target_condition: The query used to define targeted devices or
     modules. The query is based on twin tags and/or reported properties.
    :type target_condition: str
    :param custom_metric_queries: The key-value pairs with queries and their
     identifier.
    :type custom_metric_queries: dict[str, str]
    """

    _attribute_map = {
        "target_condition": {"key": "targetCondition", "type": "str"},
        "custom_metric_queries": {"key": "customMetricQueries", "type": "{str}"},
    }

    def __init__(
        self, *, target_condition: str = None, custom_metric_queries=None, **kwargs
    ) -> None:
        super(ConfigurationQueriesTestInput, self).__init__(**kwargs)
        self.target_condition = target_condition
        self.custom_metric_queries = custom_metric_queries
