# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from .iot_hub_gateway_service_ap_is import IotHubGatewayServiceAPIs
from .version import VERSION

__all__ = ["IotHubGatewayServiceAPIs"]

__version__ = VERSION
